from click import (
    Argument,
    Option,
    Parameter
)
from click.exceptions import UsageError, MissingParameter


class OpinionatedParameter(Parameter):
    """
    Custom class for parameters that are required or forbidden depending on
    the other options.
    """
    def __init__(self, *args, **kwargs):
        self.required_with = kwargs.pop('required_with', None)
        self.required_without = kwargs.pop('required_without', None)
        self.forbidden_with = kwargs.pop('forbidden_with', None)
        return super(OpinionatedParameter, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        """Raise exceptions if the opinions of the parameter are not met."""
        if self.required_with and opts.get(self.name) is None:
            for opt in self.required_with:
                if opts.get(opt):
                    raise MissingParameter(param=self, ctx=ctx)
        if self.required_without and opts.get(self.name) is None:
            if all(opts.get(opt) is None for opt in self.required_without):
                raise MissingParameter(param=self, ctx=ctx)
        if self.forbidden_with and opts.get(self.name) is not None:
            if any(opts.get(opt) is not None for opt in self.forbidden_with):
                raise UsageError(u'{} is not allowed together with {}.'.format(
                    _format_option(self.name),
                    ', '.join(
                        _format_option(opt) for opt in self.forbidden_with
                    )
                ))
        return super(OpinionatedParameter, self).handle_parse_result(
            ctx,
            opts,
            args
        )


class OpinionatedOption(OpinionatedParameter, Option):
    """
    Custom class for options that are required or forbidden depending on the
    other options.
    """
    pass


class OpinionatedArgument(OpinionatedParameter, Argument):
    """
    Custom class for arguments that are required or forbidden depending on the
    other options.
    """
    pass


def _format_option(opt):
    """Format an option name in its command line format."""
    return u'--{}'.format(opt.replace(u'_', u'-'))
