#!/usr/bin/env python3

"""
The check_vmware CLI program that can run checks against a vSphere API host.
Based on the Click_ and nagiosplugin_ libraries.

.. _Click: http://click.pocoo.org
.. _nagiosplugin: https://bitbucket.org/flyingcircus/nagiosplugin
"""

from sys import stdout
from traceback import format_exc

from click import (
    Choice,
    argument,
    command,
    echo,
    option,
    wrap_text
)
from click.exceptions import ClickException, UsageError
from nagiosplugin.error import CheckError

from check_vmware.cli.helpers import (
    OpinionatedOption,
    OpinionatedArgument
)
from check_vmware.cli.resources import ServiceResources
from check_vmware.cli.nagios import check


def main():
    try:
        cli(standalone_mode=False)
    except ClickException as e:
        e.show(file=stdout)
        exit(3)
    except CheckError as e:
        # CheckError are normally handled by nagiosplugin, but we need to
        # cover it here too. E.g. the --list-counters option does not go
        # through the nagiousplugin but can also raise CheckErrors.
        echo('Error: {}'.format(e))
        exit(3)
    except Exception:
        echo(format_exc())
        exit(3)


@command(context_settings={'help_option_names': ['-h', '--help']})
@option(
    '--service-host',
    default='http://localhost',
    show_default=True,
    help=u'Host name of the plugin service.'
)
@option(
    '--service-port',
    type=int,
    default=8000,
    show_default=True,
    help=u'Port of the plugin service.'
)
@option(
    '-H',
    '--host',
    help=(
        u'Host name of the vSphere API. This option is required unless the '
        u'service is configured for exactly one vSphere host.'
    )
)
@option(
    '-o',
    '--port',
    type=int,
    default=443,
    show_default=True,
    help=u'Port of the vSphere API.'
)
@option(
    '--ignore-ssl',
    is_flag=True,
    default=False,
    show_default=True,
    help=u'Use this if the vSphere API has an invalid SSL certificate.'
)
@option(
    '-u',
    '--username',
    help=(
        u'Your vSphere API username. This option is required unless the '
        u'service has a configured username for the given host.'
    )
)
@option(
    '-p',
    '--password',
    help=(
        u'Your vSphere API password. This option is required unless the '
        u'service has a configured password for the given host.'
    )
)
@option(
    '-D',
    '--datacenter',
    help=(
        u'Name of the target datacenter. If used together with the --cluster, '
        u'--host-system or --vm options, it will be used as a filter.'
    )
)
@option(
    '-C',
    '--cluster',
    help=(
        u'Name of the target cluster. If used together with the '
        u'--vm or --host-system options, it will be used as a filter.'
    )
)
@option(
    '-S',
    '--host-system',
    help=(
        u'Name of the target ESXi host system.'
    )
)
@option(
    '-V',
    '--vm',
    help=u'Name of the target virtual machine.'
)
@option(
    '-t',
    '--entity-type',
    type=Choice(['vm', 'host', 'cluster', 'datacenter']),
    cls=OpinionatedOption,
    required_with=['entity_name'],
    help=(
        u'Type of the target managed entity. Use together with --entity-name '
        u'as an alternative to the --datacenter, --cluster, --host-system and '
        u'--vm options.'
    )
)
@option(
    '-n',
    '--entity-name',
    cls=OpinionatedOption,
    required_with=['entity_type'],
    help=(
        u'Name of the target managed entity. Use together with --entity-type '
        u'as an alternative to the --datacenter, --cluster, --host-system and '
        u'--vm options.'
    )
)
@option(
    '-x',
    '--ignore-result',
    multiple=True,
    help=(
        u'Ignore all results with a matching description. This option can be '
        u'repeated to ignore multiple type of results. (Note that the --regex '
        u'option does not affect --ignore-result.)'
    )
)
@option(
    '-r',
    '--regex',
    is_flag=True,
    help=(
        u'Treat any optional part of a counter as a regular expression that '
        u'includes items matched by the expression. Counters where --regex is '
        u'a valid option are listed by --list-counters -vv'
    )
)
@option(
    '-b',
    '--blacklist',
    is_flag=True,
    help=(
        u'Treat any optional part of a counter as a blacklist that excludes '
        u'matching items, rather than excluding non-matching items. Counters '
        u'where --blacklist is a valid option are listed by '
        u'--list-counters -vv'
    )
)
@option(
    '-i',
    '--interval',
    type=int,
    help=(
        u'The sampling period interval in seconds.  [default: 20 for vm and '
        u'host target entities, 300 for datacenter and cluster target '
        u'entities]'
    )
)
@option(
    '--instance',
    default='',
    show_default=False,
    help=u'The counter instance to use.  [default: ""]'
)
@option(
    '--timeout',
    type=float,
    default=10.0,
    show_default=True,
    help=u'Timeout in seconds.'
)
@option(
    '--list-counters',
    cls=OpinionatedOption,
    forbidden_with=['list_all_counters'],
    is_flag=True,
    default=False,
    help=(
        u'List all available counters for the specific entity and interval. '
        u'Add -v to print counter summaries. Add -vv to print available '
        u'counter instances.'
    )
)
@option(
    '--list-all-counters',
    cls=OpinionatedOption,
    forbidden_with=[
        'entity_name', 'entity_type', 'datacenter', 'cluster', 'host_system',
        'vm', 'list_counters'
    ],
    is_flag=True,
    default=False,
    help=(
        u'List all counters known by the vSphere server. Add -v to print '
        u'counter summaries.'
    )
)
@option(
    '-v',
    '--verbose',
    count=True,
    help=(
        u'Print more detailed information. Adding multiple instances of this '
        u'flag further increases the level of verbosity. Level 1 (-v) prints '
        u'more detailed information about the most significant metrics (those '
        u'with the "worst" state), but prints nothings extra if all metrics '
        u'are OK. Level 2 (-vv) prints more detailed information about the '
        u'most significant metrics, also if all metrics are OK (which is then '
        u'the most significant state). Level 3 (-vvv) prints detailed '
        u'information for all metrics regardless of their state.'
    )
)
@option(
    '-c',
    '--critical',
    help=u'Set critical threshold.'
)
@option(
    '-w',
    '--warning',
    help=u'Set warning threshold.'
)
@argument(
    'counter',
    nargs=1,
    cls=OpinionatedArgument,
    required_without=['list_counters', 'list_all_counters'],
    required=False
)
def cli(**params):
    """
    Query performance data from a vSphere API server.

    Specify the managed entity and performance counter to query.

    If no managed entity is specified, the query will be made against either
    the vCenter Server Appliance virtual machine, or the ESXi host, depending
    on whether the targetted vSphere API server is a vCenter or an ESXi host.

    Below is a list of commonly used performance counters and the types of
    entities they are available on. Use the --list-counters option to fetch a
    full list of available performance counters. Use the --verbose option to
    list more information about the fetched counters.

    \b
    cpu.usage.average (datacenter, cluster, host, vm)
    disk.usage.average (host, vm)
    disk.read.average (host, vm)
    disk.write.average (host, vm)
    mem.usage.average (cluster, host, vm)
    net.usage.average (cluster, host, vm)
    service.status (host)
    service.status.<service_name> (host)
    runtime.connected (host, vm)
    runtime.status (datacenter, cluster, host, vm)
    vmfs.usage (datacenter, cluster, host)
    vmfs.usage.<datastore_name> (datacenter, cluster, host)

    The following counters can be useful for one-off checks, but should be
    avoided in scheduled use since they are very performance demanding, both
    for Monitor and for the vCenter server being monitored:

    \b
    runtime.list_vms (datacenter, cluster, host)
    runtime.list_hosts (datacenter, cluster)
    runtime.tools (datacenter, cluster, host, vm)
    device.cddvd (datacenter, cluster, host)
    """

    # Translate the entity_type and entity_name to the corresponding
    # datacenter, cluster, host_system or vm option.
    if 'entity_name' in params:
        entity_type = params.pop('entity_type')
        if entity_type == 'host':
            entity_type = 'host_system'
        params[entity_type] = params.pop('entity_name')

    # Do some last minute validation, that's more easily done after the
    # --entity-* options have been translated.
    if params.get('host_system') and params.get('vm'):
        raise UsageError(u'--host-system option cannot be used as a filter.')

    resources = ServiceResources(
        params['service_host'],
        params['service_port'],
        params['timeout']
    )

    if params['list_counters']:
        query_params = {
            k: params[k] for k in (
                'host',
                'port',
                'ignore_ssl',
                'username',
                'password',
                'datacenter',
                'cluster',
                'host_system',
                'vm',
                'interval'
            )
        }
        print_counters(
            resources.list_counters(query_params),
            params['counter'],
            verbose=params['verbose']
        )
        exit(0)
    elif params['list_all_counters']:
        query_params = {
            k: params[k] for k in (
                'host',
                'port',
                'ignore_ssl',
                'username',
                'password'
            )
        }
        print_counters(
            resources.list_all_counters(query_params),
            params['counter'],
            verbose=params['verbose']
        )
        exit(0)
    else:
        query_params = {
            k: params[k] for k in (
                'host',
                'port',
                'ignore_ssl',
                'username',
                'password',
                'datacenter',
                'cluster',
                'host_system',
                'vm',
                'counter',
                'interval',
                'instance',
                'regex',
                'blacklist'
            )
        }
        check(
            resources,
            query_params,
            params['warning'],
            params['critical'],
            params['verbose'],
            ignore_result=params['ignore_result'],
        )
        # check() will exit for us with the appropiate exit code


def print_counters(data, counter, verbose=0):
    ind1 = ' ' * 4
    ind2 = ' ' * (15 if verbose >= 2 else 4)
    for counter_name, counter_data in sorted(data['data'].items()):
        if counter and not counter_name.startswith(counter):
            continue
        echo(counter_name)
        if verbose >= 1:
            summary = counter_data.get(
                'summary',
                u'No help available.'
            ).strip()
            if verbose >= 2:
                summary = u'Summary:   {}'.format(summary)
            echo(wrap_text(
                summary,
                initial_indent=ind1,
                subsequent_indent=ind2
            ))
            flags = counter_data.get('flags')
            if verbose >= 2 and flags:
                echo(ind1 + u'Flags:     {}'.format(', '.join(flags)))
            if verbose >= 2 and 'instances' in counter_data:
                echo(ind1 + u'Instances: {}'.format(
                    ('\n' + ind2).join(sorted(counter_data['instances']))
                ))


if __name__ == '__main__':
    main()
