from collections import namedtuple

from nagiosplugin import (
    Resource,
    Check,
    Metric,
    ScalarContext,
    guarded,
    Context,
    Summary,
    Ok, Warn, Critical, Unknown
)
from nagiosplugin.state import ServiceState


class Ignored(ServiceState):
    """
    Special state class for ignored results. The code of -1 will make it the
    least significant state.
    """
    def __new__(cls):
        return super(cls, Ignored).__new__(cls, -1, 'ignored')


Ignored = Ignored()


class StateContext(Context):
    """
    Nagios context class for data that contains a state.
    """

    def __init__(self, name, **kwargs):
        fmt_metric = u'{name}: {value[1]}'
        super(StateContext, self).__init__(
            name,
            fmt_metric=fmt_metric,
            **kwargs
        )

    def evaluate(self, metric, resource):
        """Evaluate the result based on its state."""
        (check_state, text) = metric.value
        if check_state == 'ok':
            state = Ok
        elif check_state == 'warning':
            state = Warn
        elif check_state == 'critical':
            state = Critical
        elif check_state == 'unknown':
            state = Unknown
        else:
            raise ValueError(u'{} is not a supported value'.format(
                check_state
            ))
        return self.result_cls(state, '', metric)


class DetailedContext(ScalarContext):
    """
    Nagios context class for data that has additional information in the
    `detail` attribute.
    """
    def describe(self, metric):
        return u'{name}: {value}{uom} ({detail})'.format(
            name=metric.name,
            value=metric.value,
            uom=metric.uom,
            detail=metric.detail
        )


class DetailedMetric(
    namedtuple('DetailedMetric', Metric._fields + ('detail',)),
    Metric
):
    """
    Nagios metric class for data that has additional information in the
    `detail` attribute.
    """
    def __new__(
        cls, name, value, uom=None, min=None, max=None, context=None,
        contextobj=None, resource=None, detail=None
    ):
        return tuple.__new__(
            cls,
            (
                name, value, uom, min, max, context or name, contextobj,
                resource, detail
            )
        )


class IgnorantContext(Context):
    """ Nagios context class for metrics that might be ignored. """

    def __init__(self, *args, ignore_result=None, **kwargs):
        super(IgnorantContext, self).__init__(*args, **kwargs)
        self.ignore_result = ignore_result

    def evaluate(self, metric, resource):
        """
        Return an `Ignored` result if the metric matches any of the ignored
        values, otherwise return a result as normal.
        """
        if (
            self.ignore_result
            and any(x in metric.description for x in self.ignore_result)
        ):
            return self.result_cls(Ignored, None, metric)
        return super(IgnorantContext, self).evaluate(metric, resource)


class IgnorantScalarContext(IgnorantContext, ScalarContext):
    """ A `ScalarContext` for metrics that might be ignored. """
    pass


class IgnorantDetailedContext(IgnorantContext, DetailedContext):
    """ A `DetailedContext` for metrics that might be ignored. """
    pass


class IgnorantStateContext(IgnorantContext, StateContext):
    """ A `StateContext` for metrics that might be ignored. """
    pass


class IgnorantCheck(Check):
    """
    Nagios check class that translates the `Ignored` state to `Ok` , if all
    results of the check would end up as `Ignored`.
    """

    @property
    def state(self):
        """
        Return the most significant state, or `Ok` if it is `Ignored`.
        """
        try:
            state = self.results.most_significant_state
            return Ok if state is Ignored else state
        except ValueError:
            return Unknown

    @property
    def exitcode(self):
        """
        Return the exitcode of our most significant state.
        """
        return int(self.state)


class CustomSummary(Summary):
    """
    Nagios summary class that implements custom output formatting.
    """

    def __init__(self, verbose):
        self.verbosity_level = verbose
        super(CustomSummary, self).__init__()

    def ok(self, results):
        """Format an OK result."""
        result_count = len(results.results)
        if result_count == 1:
            return u'{}'.format(results[0])
        ignored_count = len(results.by_state[Ignored])
        if ignored_count:
            return u'{}/{} ignored, {}/{} ok'.format(
                ignored_count,
                result_count,
                result_count - ignored_count,
                result_count,
            )
        return u'all {} results are ok'.format(result_count)

    def problem(self, results):
        """Format a non-OK result."""
        result_count = len(results.results)
        if result_count == 1:
            return str(results.first_significant)
        return u'{}: {}'.format(
            u', '.join(
                u'{}/{} {}'.format(len(v), result_count, k)
                for k, v in sorted(results.by_state.items())
            ),
            str(results.first_significant)
        )

    def verbose(self, results):
        """Print additional output based on our verbosity level."""
        msgs = []
        if not results:
            return msgs
        # If all checks are Ok, require verbose level 2 to print them all out
        if self.verbosity_level == 1 and results.most_significant_state == Ok:
            pass
        elif self.verbosity_level <= 2:
            for result in results.by_state[results.most_significant_state]:
                msgs.append(u'{0}: {1}'.format(result.state, result))
        elif self.verbosity_level == 3:
            for severity in [Critical, Warn, Unknown, Ok, Ignored]:
                for result in results.by_state.get(severity, []):
                    msgs.append(u'{0}: {1}'.format(result.state, result))
        return msgs


# This camel snake case is on purpose. Naming this class seems to
# be the only way to make the output as wanted.
class Check_Vmware(Resource):
    """
    Nagios resource class for performance data.
    """

    def __init__(self, resources, query_params):
        self.resources = resources
        self.query_params = query_params
        return

    def probe(self):
        """Query the resource for performance data."""
        data = self.resources.query(self.query_params)
        for result in data['data']:
            if 'state' in result:
                yield Metric(
                    result['name'],
                    (result['state'], result['value']),
                    context='state'
                )
            elif 'detail' in result:
                yield DetailedMetric(
                    result['name'],
                    result['value'],
                    uom=self._get_uom(result),
                    context='detail',
                    detail=result['detail']
                )
            else:
                yield Metric(
                    result['name'],
                    result['value'],
                    uom=self._get_uom(result),
                    context='scalar'
                )

    @staticmethod
    def _get_uom(result):
        uom = result.get('unit')
        if uom:
            cmp_uom = uom.lower()
            if cmp_uom == 'percent':
                return '%'
            if cmp_uom == 'num':
                return None
        return uom


@guarded(verbose=0)
def check(
    resources,
    query_params,
    warning,
    critical,
    verbose,
    ignore_result=None,
):
    """
    Setup a Nagios check for the given parameters.
    """
    check = IgnorantCheck(
        Check_Vmware(resources, query_params),
        IgnorantScalarContext(
            'scalar',
            warning=warning,
            critical=critical,
            ignore_result=ignore_result,
        ),
        IgnorantDetailedContext(
            'detail',
            warning=warning,
            critical=critical,
            ignore_result=ignore_result,
        ),
        IgnorantStateContext(
            'state',
            ignore_result=ignore_result,
        ),
        CustomSummary(verbose=verbose)
    )
    check.main(verbose=verbose)
