from requests import get
from requests.exceptions import (
    ConnectionError as RequestsConnectionError,
    InvalidSchema,
    ReadTimeout,
)
from nagiosplugin.error import CheckError

from check_vmware.cli.exceptions import (
    InputException,
    ServiceException,
    ServiceConnectionException
)


class ServiceResources(object):
    """
    Resource class that fetch data from the service HTTP API.
    """

    def __init__(self, service_host, service_port, timeout):
        self.service_host = service_host
        self.service_port = service_port
        self.timeout = timeout

    def query(self, query_params):
        """Query the service for performance data."""
        service_url = '{}:{}/{}'.format(
            self.service_host,
            self.service_port,
            'v1/query'
        )
        return self._query_service(service_url, query_params)

    def list_counters(self, query_params):
        """
        Query the service for counters available on the given entity.
        """
        service_url = '{}:{}/{}'.format(
            self.service_host,
            self.service_port,
            'v1/list-counters'
        )
        return self._query_service(service_url, query_params)

    def list_all_counters(self, query_params):
        """
        Query the service for all counters.
        """
        service_url = '{}:{}/{}'.format(
            self.service_host,
            self.service_port,
            'v1/list-all-counters'
        )
        return self._query_service(service_url, query_params)

    def _query_service(self, service_url, query_params):
        """
        Query the service and handle any errors.
        """
        try:
            response = get(
                service_url,
                params=query_params,
                timeout=self.timeout
            )
            data = response.json()
            self._handle_errors(data, response.status_code)
            return data
        except RequestsConnectionError:
            raise ServiceConnectionException(
                u'Make sure the Check VMware plugin service is running'
            )
        except InvalidSchema:
            raise ServiceConnectionException(
                u'Make sure you specified --service-host with http:// or '
                u'https://'
            )
        except ReadTimeout:
            raise CheckError(u'Request timed out')

    def _handle_errors(self, data, status_code):
        """
        Handle any errors in the received data.
        """
        if 'errors' in data:
            for error in data['errors']:
                if status_code == 400:
                    exc = InputException
                elif status_code in [403, 404]:
                    exc = CheckError
                else:
                    exc = ServiceException
                raise exc(error['detail'])
