from flask import Blueprint, jsonify
from schematics.exceptions import BaseError as SchematicsBaseError
from werkzeug.exceptions import (
    BadRequest,
    Forbidden,
    NotFound,
    InternalServerError
)
from werkzeug.http import HTTP_STATUS_CODES

from check_vmware.service import logger
from check_vmware.vsphere.exceptions import (
    VsphereConnectionError,
    VsphereEntityNotFound,
    VsphereEntityNotSpecified,
    VsphereInvalidLogin,
    VsphereVmNotPoweredOn,
    VsphereCounterNotFound,
    VsphereCounterNotAvailable,
    VsphereCounterNotConfigured,
    VsphereMissingCounterInstance,
    VsphereUnauthorized,
    VsphereParameterInvalid,
    VsphereNotFound,
    VsphereRedisError
)


errorhandler = Blueprint("errorhandler", __name__)


@errorhandler.app_errorhandler(BadRequest)
@errorhandler.app_errorhandler(VsphereEntityNotSpecified)
@errorhandler.app_errorhandler(VsphereParameterInvalid)
@errorhandler.app_errorhandler(VsphereMissingCounterInstance)
@errorhandler.app_errorhandler(NotImplementedError)
def vsphere_400_handler(e):
    """Exception handler for BadRequest-type exceptions."""
    logger.info("400 error: %s", e)
    return _format_error(e, BadRequest.code)


@errorhandler.app_errorhandler(Forbidden)
@errorhandler.app_errorhandler(VsphereInvalidLogin)
@errorhandler.app_errorhandler(VsphereUnauthorized)
def vsphere_403_handler(e):
    """Exception handler for Forbidden-type exceptions."""
    logger.info("403 error: %s", e)
    return _format_error(e, Forbidden.code)


@errorhandler.app_errorhandler(NotFound)
@errorhandler.app_errorhandler(VsphereCounterNotAvailable)
@errorhandler.app_errorhandler(VsphereCounterNotConfigured)
@errorhandler.app_errorhandler(VsphereEntityNotFound)
@errorhandler.app_errorhandler(VsphereVmNotPoweredOn)
@errorhandler.app_errorhandler(VsphereCounterNotFound)
@errorhandler.app_errorhandler(VsphereNotFound)
def vsphere_404_handler(e):
    """Exception handler for Not Found-type exceptions."""
    logger.info("404 error: %s", e)
    return _format_error(e, NotFound.code)


@errorhandler.app_errorhandler(VsphereRedisError)
@errorhandler.app_errorhandler(VsphereConnectionError)
@errorhandler.app_errorhandler(InternalServerError)
@errorhandler.app_errorhandler(Exception)
def vsphere_500_handler(e):
    """Exception handler for Internal Error-type exceptions."""
    logger.error("500 error: %s", e, exc_info=e)
    return _format_error(e, InternalServerError.code)


@errorhandler.app_errorhandler(SchematicsBaseError)
def schematics_handler(e):
    """
    Exception handler for schematics exceptions raised when validating
    user input.

    Return multiple errors if there were several validation errors.
    """

    logger.info("400 error: %s", e)
    errors = []
    for field, field_errors in e.to_primitive().items():
        if not isinstance(field_errors, list):
            field_errors = [field_errors]
        for field_error in field_errors:
            errors.append({
                'detail': u'{}: {}'.format(field, field_error),
                'status': _get_status_message(BadRequest.code)
            })
    return (
        jsonify(dict(errors=errors)),
        BadRequest.code
    )


def _format_error(e, code):
    return (
        jsonify({
            'errors': [{
                'detail': u'{}'.format(e),
                'status': _get_status_message(code)
            }]
        }),
        code
    )


def _get_status_message(code):
    return u'{} {}'.format(code, HTTP_STATUS_CODES.get(code, 'Unknown Error'))
