from functools import wraps

from flask import request, current_app
from schematics.models import Model
from schematics.types import (
    StringType,
    IntType,
    BooleanType
)
from schematics.exceptions import ModelValidationError

from check_vmware.vsphere.constants import ENTITY_TYPES


def validate(schema):
    """Decorator for validating Flask routes with Schematics models."""
    def decorator(f):
        @wraps(f)
        def wrapper(*_, **kwargs):
            data = request.args.to_dict()
            data.update(kwargs)
            params = schema(data)
            params.validate()
            return f(params)
        return wrapper
    return decorator


class ConnectionParams(Model):
    """Base schema for parameter models that connects to a vSphere API."""

    host = StringType(required=True)
    port = IntType(default=443)
    username = StringType(required=True)
    password = StringType(required=True)
    ignore_ssl = BooleanType(default=False)

    def validate(self, *args, **kwargs):
        """
        Patch up missing data from our SERVERS config before validation.
        """
        servers = current_app.config.get('SERVERS')
        if servers:
            host = self.get('host')
            config = None
            if host:
                config = servers.get(host)
            elif len(servers) == 1:
                self['host'], config = next(iter(servers.items()))
            if config:
                for k, v in config.items():
                    if self.get(k) in (None, False):
                        self[k] = v
        return Model.validate(self, *args, **kwargs)


class EntityParams(ConnectionParams):
    """
    Base schema for parameter models that targets a managed entity.
    """

    datacenter = StringType()
    cluster = StringType()
    host_system = StringType()
    vm = StringType()
    interval = IntType(default=0)  # Actual default is based on entity type

    @property
    def entity_path(self):
        """
        Return a list of tuples with the type and name of the entities leading
        up to the target entity. The actual target entity being the last tuple.

        Note that we translate the entity type from 'host_system' to 'host' for
        easier fit with the backend code that use this.
        """
        return [
            (k, getattr(self, 'host_system' if k == 'host' else k))
            for k in reversed(list(ENTITY_TYPES.keys()))
            if getattr(self, 'host_system' if k == 'host' else k, None)
        ]

    def validate(self, *args, **kwargs):
        """
        Validate that we can build a correct entity path and set sensible
        defaults.
        """
        if self.host_system and self.vm:
            raise ModelValidationError(
                {'host_system': u"cannot be used as a filter."}
            )
        if self.interval == 0:
            try:
                if self.entity_path[-1][0] in ('host', 'vm'):
                    self.interval = 20
                else:
                    self.interval = 300
            except IndexError:
                self.interval = 20
        return ConnectionParams.validate(self, *args, **kwargs)


class ListCountersParams(EntityParams):
    """Schema for valid list-counters parameters."""
    pass


class ListAllCountersParams(ConnectionParams):
    """Schema for valid list-all-counters parameters."""
    pass


class QueryParams(EntityParams):
    """Schema for valid query parameters."""
    counter = StringType(required=True)
    regex = BooleanType()
    blacklist = BooleanType()
    instance = StringType()

    def validate(self, *args, **kwargs):
        """
        Adapt the schema to allow an `counter:instance` syntax, since that is
        how it is displayed.
        """
        if self.counter and ':' in self.counter:
            self.counter, self.instance = self.counter.rsplit(':', 1)
        return EntityParams.validate(self, *args, **kwargs)
