from pyVmomi import vim, vmodl
from check_vmware.vsphere.counters import (
    find_counter,
    is_available_counter
)
from check_vmware.vsphere.entities import (
    invalidate_entity_cache,
    get_target_entity_name
)
from check_vmware.vsphere.exceptions import (
    VsphereCounterNotAvailable,
    VsphereEntityNotFound,
    VsphereMissingCounterInstance
)


def query(
        connection,
        entity,
        entity_path,
        counter_name,
        interval,
        instance
):
    """
    Return performance data for the given entity and performance counter.
    """

    counter = find_counter(connection, counter_name)

    metrics = [
        vim.PerformanceManager.MetricId(
            counterId=counter['id'],
            instance=instance
        )
    ]

    queries = [
        vim.PerformanceManager.QuerySpec(
            intervalId=interval,
            entity=entity,
            metricId=metrics,
            maxSample=1
        )
    ]

    # If the counter does not exist on the entity, the vSphere API
    # usually just returns an empty result, but sometimes it returns
    # an InvalidArgument exception here
    try:
        results = connection.content.perfManager.QueryPerf(querySpec=queries)
    except vmodl.fault.InvalidArgument as e:
        if e.invalidProperty == 'metricId':
            raise VsphereCounterNotAvailable(
                counter_name,
                get_target_entity_name(entity_path)
            )
        raise

    data = []

    if results:
        _parse_results(results, data, counter)

    if not data:
        try:
            if not is_available_counter(
                    connection,
                    counter['id'],
                    entity,
                    interval
            ):
                raise VsphereCounterNotAvailable(
                    counter_name,
                    get_target_entity_name(entity_path)
                )
            else:
                # It is *probably* a problem with the instance at this point
                raise VsphereMissingCounterInstance(counter_name, instance)
        except vmodl.fault.InvalidArgument:
            # This probably means that the entity moved or disappeared.
            invalidate_entity_cache(connection, entity_path)
            raise VsphereEntityNotFound(get_target_entity_name(entity_path))

    return dict(data=data)


def _parse_results(results, data, counter):
    for entity_results in results:
        for counter_results in entity_results.value:
            # We're only asking for one sample -- the latest
            sample = counter_results.value[-1]
            assert counter_results.id.counterId == counter['id']
            if counter['unit'] == '%':
                sample = round(sample * 0.01, 2)
            instance = counter_results.id.instance
            instance_suffix = instance and u':{}'.format(instance)
            data.append({
                'name': counter['name'] + instance_suffix,
                'unit': counter['unit'],
                'value': sample
            })
