from pyVmomi import vim
from check_vmware.vsphere.helpers import implemented_for
from check_vmware.vsphere.entities import list_entities
from check_vmware.vsphere.exceptions import (
    VsphereException, VsphereCounterNotFound
)


@implemented_for(['vm', 'host'])
def connected(connection, entity, _flags):
    """
    Return the connection state of the given entity. The returned state is one
    of: `ok` or `critical`.
    """
    connection_state = entity.runtime.connectionState
    state = 'ok' if connection_state == 'connected' else 'critical'
    return [{
        'name': 'connection state',
        'value': connection_state,
        'state': state
    }]


@implemented_for(['vm'])
def cpu(connection, entity, _flags):
    """
    Return current upper-bound on CPU usage of the given VM.
    """
    return [{
        'name': 'cpu usage',
        'value': entity.runtime.maxCpuUsage,
        'unit': 'MHz'
    }]


@implemented_for(['vm'])
def mem(connection, entity, _flags):
    """
    Return current upper-bound on memory usage of the given VM.
    """
    return [{
        'name': 'memory usage',
        'value': entity.runtime.maxMemoryUsage,
        'unit': 'MB'
    }]


@implemented_for(['vm'])
def powered_on(connection, entity, _flags):
    """
    Return the current power state of the given VM. The returned state is one
    of: `ok` or `critical`.
    """
    power_state = entity.runtime.powerState
    state = 'ok' if power_state == 'poweredOn' else 'critical'
    return [{
        'name': 'power state',
        'value': power_state,
        'state': state
    }]


@implemented_for(['vm', 'host', 'cluster', 'datacenter'])
def status(connection, entity, _flags):
    """
    Return the general status of the given entity. The returned state is one
    of: `ok`, `warning`, `critical` or `unknown`.
    """
    status = entity.overallStatus.lower()
    status_text = 'status is {}'.format(status)
    severity = _colour_to_state(status)
    return [{
        'name': entity.name,
        'value': status_text,
        'state': severity
    }]


@implemented_for(['vm'])
def console_connections(connection, entity, _flags):
    """
    Return the number of active MKS connections of the given VM.
    """
    return [{
        'name': 'console connections',
        'value': entity.runtime.numMksConnections
    }]


@implemented_for(['vm'])
def guest_state(connection, entity, _flags):
    """
    Return the general state of the guest operating system of the given VM.
    The returned state is one of: `ok` or `critical`.
    """
    guest_state = entity.guest.guestState
    state = 'ok' if guest_state == 'running' else 'critical'
    return [{
        'name': 'guest state',
        'value': guest_state,
        'state': state
    }]


@implemented_for(['vm', 'host', 'cluster', 'datacenter'])
def tools(connection, entity, _flags):
    """
    Return the general state of VMware Tools in the guest operating system of
    the given entity. The returned state is one of: `ok`, `warning` or
    `critical`.
    """
    if isinstance(entity, vim.VirtualMachine):
        vm_list = [entity]
    else:
        vm_list = list_entities(connection, entity, 'vm')
    items = []
    for vm in vm_list:
        tools_running = vm.guest.toolsRunningStatus
        tools_running = tools_running.split('guestTools', 1)[1]
        tools_version = vm.guest.toolsVersionStatus
        tools_version = tools_version.split('guestTools', 1)[1]
        if tools_running == 'Running':
            if tools_version in ['Newest', 'Unmanaged', 'Current']:
                state = 'ok'
            else:
                state = 'warning'
        else:
            state = 'critical'
        value = u'status: {}, version: {}'.format(tools_running, tools_version)
        items.append({
            'name': vm.name,
            'value': value,
            'state': state
        })
    return items


@implemented_for(['vm', 'host', 'cluster', 'datacenter'])
def issues(connection, entity, _flags):
    """
    Return current configuration issues that have been detected for this
    entity. The returned state is one of: `ok`, `warning` or `critical`.
    """
    items = []
    for issue in entity.configIssue:
        if getattr(issue, 'severity', None) in ('user', 'info'):
            state = 'warning'
        else:
            state = 'critical'
        items.append({
            'name': 'issues',
            'value': issue.fullFormattedMessage,
            'state': state
        })
    if not items:
        return [{
            'name': 'issues',
            'value': 'No config issues',
            'state': 'ok'
        }]
    return items


@implemented_for(['host'])
def health(connection, entity, _flags):
    """
    Return general hardware and system health of the given host. The returned
    state is one of: `ok`, `warning`, `critical` or `unknown`.
    """
    hardware = entity.runtime.healthSystemRuntime.hardwareStatusInfo
    system = entity.runtime.healthSystemRuntime.systemHealthInfo
    check = [('cpu', hardware.cpuStatusInfo),
             ('storage', hardware.storageStatusInfo),
             ('memory', hardware.memoryStatusInfo)]
    items = []
    for item_type, path in check:
        for item in path:
            status = item.status
            items.append({
                'name': item_type,
                'value': u'{}: {}'.format(item.name, status.summary),
                'state': _colour_to_state(status.key)
            })
    for item in system.numericSensorInfo:
        status = item.healthState
        items.append({
            'name': item.sensorType,
            'value': u'{}: {}'.format(item.name, status.summary),
            'state': _colour_to_state(status.key)
        })
    return items


@implemented_for(['host'])
def maintenance(connection, entity, _flags):
    """
    Return the maintenance status of the given host. The returned state is
    `ok` if the host is not in maintence mode and `critical` if it is.
    """
    if entity.runtime.inMaintenanceMode:
        state = 'critical'
        value = u'{} needs maintenance'.format(entity.name)
    else:
        state = 'ok'
        value = u'{} doesn\'t need maintenance'.format(entity.name)
    return [{
        'name': 'maintenance',
        'value': value,
        'state': state
    }]


@implemented_for(['host', 'cluster', 'datacenter'])
def list_vms(connection, entity, _flags):
    """
    Return all VMs of the given entity and the current power state for each
    returned VM.
    """
    items = []
    for vm in list_entities(connection, entity, 'vm'):
        state = 'ok' if vm.runtime.powerState == 'poweredOn' else 'warning'
        items.append({
            'name': vm.name,
            'value': vm.runtime.powerState,
            'state': state
        })
    return items


@implemented_for(['cluster', 'datacenter'])
def list_hosts(connection, entity, _flags):
    """
    Return all hosts of the given entity and the current power state for each
    returned host.
    """
    items = []
    for host in list_entities(connection, entity, 'host'):
        state = 'ok' if host.runtime.powerState == 'poweredOn' else 'warning'
        items.append({
            'name': host.name,
            'value': host.runtime.powerState,
            'state': state
        })
    return items


@implemented_for(['datacenter'])
def list_clusters(connection, entity, _flags):
    """
    Return all clusters of the given datacenter and a host and status summary
    for each returned cluster.
    """
    items = []
    for cluster in list_entities(connection, entity, 'cluster'):
        value = u'status is {} and it has {} hosts'.format(
            cluster.overallStatus,
            cluster.summary.numHosts
        )
        items.append({
            'name': cluster.name,
            'value': value,
            'state': _colour_to_state(cluster.overallStatus)
        })
    return items


@implemented_for(['datacenter'])
def recommendations(connection, entity, _flags):
    """
    Return all clusters of the given datacenter and a summary of migration
    recommendations for each returned cluster.
    """
    items = []
    for cluster in list_entities(connection, entity, 'cluster'):
        for recommendation in cluster.recommendation:
            value = u'({}) {}: {}'.format(
                recommendation.rating,
                recommendation.reason,
                recommendation.reasonText
            )
            items.append({
                'name': cluster.name,
                'value': value,
                'state': 'warning'
            })
    if items:
        return items
    else:
        return [{
            'name': entity.name,
            'value': 'No recommendations',
            'state': 'ok'
        }]


@implemented_for(['vm', 'host', 'cluster', 'datacenter'])
def alarms(connection, entity, _flags, mode=None):
    """
    Return any alarms raised by VMware for the given entity and
    any of its descendants. Alarms acknowledged on the VMware side
    are considered OK unless the 'all' mode is given.
    """
    if mode not in [None, 'all']:
        raise VsphereCounterNotFound('runtime.alarms.{}'.format(mode))
    alarms = entity.triggeredAlarmState
    if not alarms:
        return [{
            'name': 'alarms',
            'value': 'no alarms',
            'state': 'ok'
        }]
    items = []
    for alarm in alarms:
        description = alarm.alarm.info.description
        if mode is None and alarm.acknowledged:
            status = 'ok'
        else:
            status = _colour_to_state(alarm.overallStatus)
        alarm_entity = alarm.entity.name
        items.append({
            'name': alarm_entity,
            'value': description,
            'state': status
        })
    return items


def _colour_to_state(colour):
    colour = colour.lower()
    if colour == 'green':
        return 'ok'
    elif colour == 'yellow':
        return 'warning'
    elif colour == 'red':
        return 'critical'
    elif colour in ('gray', 'unknown'):
        return 'unknown'
    else:
        raise VsphereException(u'{} not a valid status'.format(colour))
