"""
The states returned by these checks are based on the Perl plugin fork by
Martin Fuerstenau:
https://github.com/BaldMansMojo/check_vmware_esx/blob/270d643145d617b54f056b74d73a8f00f049f473/modules/host_storage_info.pm  # NOQA
"""

from check_vmware.vsphere.helpers import (
    implemented_for,
    matches,
    function_properties
)
from check_vmware.vsphere.exceptions import VsphereNotFound


@function_properties(counter_flags=['regex', 'blacklist'])
@implemented_for(['host'])
def adapter(connection, entity, flags, adapter_name=None):
    """
    Return a list of the available host bus adapters of the given host and a
    summary of the online status of each adapter. Specify <adapter_name> to
    include only matching adapters. You may specify a comma-separated list of
    adapters.
    """
    device = entity.configManager.storageSystem.storageDeviceInfo
    adapters = device.hostBusAdapter
    items = []
    for adapter in adapters:
        if not matches(adapter.device, adapter_name, flags):
            continue
        status = adapter.status.lower()
        if status == 'online':
            state = 'ok'
            status_string = 'adapter is online'
        elif status == 'offline':
            state = 'critical'
            status_string = 'adapter is offline'
        elif status == 'unbound':
            state = 'warning'
            status_string = 'adapter is unbound'
        elif status == 'unknown':
            state = 'warning'
            status_string = 'adapter state is unknown'
        else:
            state = 'unknown'
            status_string = u'state is {}'.format(status)
        items.append({
            'name': adapter.device,
            'value': status_string,
            'state': state
        })
    if adapter_name and not items:
        raise VsphereNotFound('adapters', adapter_name)
    return items


@function_properties(counter_flags=['regex', 'blacklist'])
@implemented_for(['host'])
def lun(connection, entity, flags, lun_name=None):
    """
    Return a list of all available SCSI logical units of the host and a
    summary of their operational state for each LUN. Specify <lun_name> to
    include only matching logical units. You may specify a comma-separated
    list of logical units.
    """
    device = entity.configManager.storageSystem.storageDeviceInfo
    lun_map = {}
    for adapter in device.scsiTopology.adapter:
        for target in adapter.target:
            for lun in target.lun:
                lun_map[lun.scsiLun] = lun.lun
    items = []
    for scsi in device.scsiLun:
        states = scsi.operationalState
        check_state = _worst_lun_state(states)
        name = scsi.key.split('-')[-1]
        if not matches(name, lun_name, flags):
            continue
        status = u'lun {} is in states "{}"'.format(
            lun_map[scsi.key],
            ', '.join(states)
        )
        items.append({
            'name': name,
            'value': status,
            'state': check_state
        })
    if lun_name and not items:
        raise VsphereNotFound('lun', lun_name)
    return items


@implemented_for(['host'])
def path(connection, entity, _flags):
    """
    Return a list of all paths to all logical units that can be configured for
    multipathing and states.

    The state returned is OK for paths that are 'active' or 'standby'. Other
    states are reported as a WARNING if there is at least one other OK path
    to that logical unit, and CRITICAL otherwise.
    """
    device = entity.configManager.storageSystem.storageDeviceInfo
    items = []
    ok_states = ('active', 'standby')
    for multipath in device.multipathInfo.lun:
        bad_state = 'critical'
        for path in multipath.path:
            if path.state in ok_states:
                bad_state = 'warning'
                break
        for n, path in enumerate(multipath.path):
            items.append({
                'name': u'{}: path {}: {}'.format(
                    multipath.id,
                    n + 1,
                    path.name or path.key or 'unnamed path'
                ),
                'state': 'ok' if path.state in ok_states else bad_state,
                'value': path.state
            })
    return items


def _worst_lun_state(lun_states):
    state_map = {
        'ok': 'ok',
        'degraded': 'warning',
        'unknownState': 'warning',
        'error': 'critical',
        'off': 'critical',
        'quiesced': 'critical',
        'timeout': 'critical'
    }
    check_states = set(state_map[lun_state] for lun_state in lun_states)
    for state in ['critical', 'warning', 'ok']:
        if state in check_states:
            return state
    return 'ok'
