from collections import Counter
import json


from check_vmware.vsphere.helpers import implemented_for

from check_vmware.vsphere.entities import list_entities
from check_vmware.vsphere.exceptions import (
    VsphereParameterInvalid,
    VsphereNotFound,
    VsphereException
)
from check_vmware.vsphere.constants import ENTITY_TYPES


HEALTH_STATES = {
    'healthy': 'ok',
    'unhealty': 'critical',
    'unknown': 'warning'
}


@implemented_for(['host', 'cluster', 'datacenter'])
def health(connection, entity, _flags):
    """
    Check the vSAN health status.
    """
    host = _one_vsan_host(connection, entity)
    cluster_status = host.configManager.vsanSystem.QueryHostStatus()
    health = cluster_status.health
    return [{
        'name': 'health',
        'value': health,
        'state': HEALTH_STATES.get(health, 'unknown')
    }]


@implemented_for(['host', 'cluster', 'datacenter'])
def disk_usage(connection, entity, _flags):
    """
    Usage of the physical disks in the vSAN.
    """
    host = _one_vsan_host(connection, entity)
    items = []
    info = _vsan_info(host, props=['capacity', 'capacityUsed'])
    for key, disk in info.items():
        if disk['capacity']:
            usage = disk['capacityUsed'] * 100.0 / disk['capacity']
            items.append({
                'name': key,
                'value': usage,
                'unit': '%'
            })
    return items


@implemented_for(['host', 'cluster', 'datacenter'])
def usage(connection, entity, _flags):
    """
    Total storage usage in the vSAN.
    """
    host = _one_vsan_host(connection, entity)
    counter = Counter()
    info = _vsan_info(host, props=['capacity', 'capacityUsed'])
    for key, disk in info.items():
        counter.update(disk)
    if not counter['capacity']:
        raise VsphereException('No vSAN disk capacity found')
    return [{
        'name': 'totalusage',
        'value': counter['capacityUsed'] * 100.0 / counter['capacity'],
        'unit': '%'
    }]


def _one_vsan_host(connection, entity):
    if isinstance(entity, ENTITY_TYPES['host']):
        if not entity.config.vsanHostConfig.enabled:
            raise VsphereParameterInvalid('Not a vSAN enabled host')
        return entity
    for host in list_entities(connection, entity, 'host'):
        if host.config.vsanHostConfig.enabled:
            return host
    raise VsphereParameterInvalid('No vSAN enabled host found')


def _vsan_info(host, disk_uuid=None, metric=None, props=None):
    vsan_internal = host.configManager.vsanInternalSystem
    json_string = vsan_internal.QueryPhysicalVsanDisks(props)
    j = json.loads(json_string)
    if not disk_uuid:
        return j
    if disk_uuid not in j:
        raise VsphereNotFound('vsan disk', disk_uuid)
    j = j[disk_uuid]
    if not metric:
        return j
    if metric not in j:
        raise VsphereNotFound('metric', metric)
    return j[metric]
