import datetime
from os.path import splitext
from json import load
from pyVmomi import vim
from check_vmware.vsphere.exceptions import (
    VsphereCounterNotConfigured,
)
from flask import current_app
from check_vmware.vsphere.constants import ENTITY_TYPES
from check_vmware.vsphere.entities import list_entities


def is_vsan_sdk_enabled():
    return current_app.config.get('VSAN_SDK_ENABLED', False)


def load_vsan_sdk_counters():
    filename, ext = splitext(__file__)
    with open('{}.json'.format(filename), 'r') as f:
        return load(f)


VSAN_SDK_COUNTERS = load_vsan_sdk_counters()


def get_vsan_sdk_counter(vsan_entity_type, metric):
    if not is_vsan_sdk_enabled:
        raise VsphereCounterNotConfigured('{}.{}.{}'.format(
            'vsansdk',
            vsan_entity_type,
            metric
        ))
    try:
        entity_counters = VSAN_SDK_COUNTERS.get(vsan_entity_type, None)
        if entity_counters:
            return entity_counters.get(metric, None)
    except ValueError:
        pass
    return None


def format_vsan_sdk_counter(vsan_entity_type, metric):
    return '{}.{}.{}'.format(
        'vsansdk',
        vsan_entity_type,
        metric
    )


def list_vsan_sdk_counters():
    return {
        format_vsan_sdk_counter(vsan_entity_type, metric): {
            'summary': metric_data.get('summary', 'No summary available.')
        }
        for vsan_entity_type, entity_data in VSAN_SDK_COUNTERS.items()
        for metric, metric_data in entity_data.items()
    }


def vsan_sdk_query(connection, entity, vsan_entity_type, metric, counter_data):
    vsan_stub = connection.vsan_stub
    pm = vim.cluster.VsanPerformanceManager(
        'vsan-performance-manager',
        vsan_stub
    )
    if isinstance(entity, ENTITY_TYPES['host']):
        vsan_entity_uuid = entity.config.vsanHostConfig.clusterInfo.uuid
        clusters = [entity.parent]
    elif isinstance(entity, ENTITY_TYPES['cluster']):
        vsan_entity_uuid = '*'
        clusters = [entity]
    else:
        vsan_entity_uuid = '*'
        clusters = list(list_entities(connection, entity, 'cluster'))
    now = datetime.datetime.utcnow()
    queries = [
        vim.cluster.VsanPerfQuerySpec(
            entityRefId='{}:{}'.format(vsan_entity_type, vsan_entity_uuid),
            # labels=[metric],  # TODO: Add this and re-record tests
            startTime=(now - datetime.timedelta(minutes=50)),
            endTime=now
        )
    ]
    counter_name = format_vsan_sdk_counter(vsan_entity_type, metric)
    items = []
    for cluster in clusters:
        csv = pm.QueryVsanPerf(cluster=cluster, querySpecs=queries)
        for page in csv:
            if len(csv) > 1 or len(clusters) > 1:
                name = u'{}: {}'.format(
                    page.entityRefId,
                    counter_name
                )
            else:
                name = counter_name
            for row in page.value:
                if row.metricId.label != metric:
                    continue
                value = int(row.values.split(',')[-1])
                unit = counter_data.get('unit')
                if unit in ('kB', 'kBps'):
                    value = float(value) / 1000
                elif unit in ('KB', 'KBps'):
                    value = float(value) / 1024
                elif unit in ('MB', 'MBps'):
                    value = float(value) / 1024 ** 2
                elif unit in ('GB', 'GBps'):
                    value = float(value) / 1024 ** 3
                elif unit == 'ms':
                    value = float(value) / 1000
                items.append({
                    'name': name,
                    'value': value,
                    'unit': unit
                })
                break
    return dict(data=items)
