import time
from flask import current_app
from check_vmware.vsphere.exceptions import VsphereCounterNotFound


def find_counter(connection, counter_name):
    """
    Return the performance counter identified by counter_name.
    """
    counter = _get_from_cache(connection, counter_name)
    if counter:
        return counter
    _maybe_refresh_cache(connection)
    counter = _get_from_cache(connection, counter_name)
    if not counter:
        raise VsphereCounterNotFound(counter_name)
    return counter


def is_available_counter(connection, counter_id, entity, interval):
    """
    Check if the given counterId is available for the given entity and
    interval.
    """
    counters = get_available_counters(
        connection.content,
        entity,
        interval
    )
    for counter in counters:
        if counter.counterId == counter_id:
            return True
    return False


def get_counters(content, include_summary=False):
    """
    Get all the available performance counters mapped to a key. The key is
    based on the counter's groupInfo, nameInfo and rollupType and correspond
    directly with the accepted 'command' input of the program.
    """
    counters = {}
    for counter in content.perfManager.perfCounter:
        counter_name = '{}.{}.{}'.format(
            counter.groupInfo.key,
            counter.nameInfo.key,
            counter.rollupType
        )
        counters[counter_name] = {
            'name': counter_name,
            'id': counter.key,
            'unit': counter.unitInfo.label
        }
        if include_summary:
            counters[counter_name]['summary'] = counter.nameInfo.summary
    return counters


def get_available_counters(content, entity, interval):
    """
    Return all vSphere counters available for a given entity and interval.
    """
    return content.perfManager.QueryAvailablePerfMetric(
        entity=entity,
        intervalId=interval
    )


def _maybe_refresh_cache(connection):
    cache_update_key = _cache_update_key(connection)
    last_update = current_app.cache.get(cache_update_key)
    now = int(time.time())
    cache_ttl = current_app.config.get('CACHE_TTL')
    if not last_update or now - last_update > cache_ttl:
        _refresh_cache(connection)
        current_app.cache.set(cache_update_key, now, timeout=0)


def _refresh_cache(connection):
    counters = get_counters(connection.content)
    for counter_name, counter in counters.items():
        _put_in_cache(connection, counter_name, counter)


def _put_in_cache(connection, counter_name, counter):
    key = _cache_key(connection, counter_name)
    current_app.cache.set(key, counter)


def _get_from_cache(connection, counter_name):
    key = _cache_key(connection, counter_name)
    value = current_app.cache.get(key)
    return value if value else None


def _cache_key(connection, counter_name):
    return u'{}_counter_{}'.format(connection.cache_key, counter_name)


def _cache_update_key(connection):
    return u'{}_counters_update_timestamp'.format(connection.cache_key)
