from flask import current_app

from check_vmware.vsphere.constants import (
    API_TYPE_HOST,
    API_TYPE_VCENTER,
    ENTITY_TYPES,
    ENTITY_TYPE_NAMES,
    POWER_STATE_ON
)
from check_vmware.vsphere.exceptions import (
    VsphereException,
    VsphereEntityNotFound,
    VsphereEntityNotSpecified,
    VsphereVmNotPoweredOn
)


def find_entity(connection, entity_path, stub):
    """
    Get the entity idenfitied by entity_path from the cache or the vSphere
    host. If the entity is cached it will be recreated using the cached
    moId and the current SessionOrientedStub.
    """
    if not entity_path:
        return _find_default_entity(connection)
    in_cache, entity = _get_from_cache(
        connection,
        entity_path,
        stub
    )
    if not in_cache:
        entity = _find_entity(connection, entity_path)
        _put_in_cache(connection, entity_path, entity)
    if not entity:
        raise VsphereEntityNotFound(
            u' / '.join(item[1] for item in entity_path)
        )
    return entity


def ensure_vm_powered_on(entity):
    """Raise a VsphereVmNotPoweredOn exception if the VM is not powered on."""
    if get_entity_type(entity) == 'vm':
        if entity.runtime.powerState != POWER_STATE_ON:
            raise VsphereVmNotPoweredOn(entity.name)


def list_entities(connection, parent, entity_type):
    """
    Yield all descendant entities of the given parent of the given entity_type.
    """
    try:
        child_class = ENTITY_TYPES[entity_type]
    except KeyError:
        raise VsphereEntityNotSpecified()
    container = connection.content.viewManager.CreateContainerView(
        parent,
        [child_class],
        recursive=True
    )
    inventory = container.view
    container.Destroy()
    for entity in inventory:
        yield entity


def get_target_entity(entity_path):
    """
    Return a tuple with the type and name of the target entity of the given
    path.
    """
    if entity_path:
        return entity_path[-1][0], entity_path[-1][1]
    return None, None


def get_target_entity_type(entity_path):
    """Return the type of the target entity of the given entity path."""
    return get_target_entity(entity_path)[0]


def get_target_entity_name(entity_path):
    """Return the name of the target entity of the given entity path."""
    return get_target_entity(entity_path)[1]


def get_entity_type(entity):
    """Return the type name of the given entity."""
    return ENTITY_TYPE_NAMES[entity.__class__]


def invalidate_entity_cache(connection, entity_path):
    """Remove any cached data for the given entity."""
    current_app.cache.delete(_cache_key(connection, entity_path))


def _get_from_cache(connection, entity_path, stub):
    key = _cache_key(connection, entity_path)
    entity_id = current_app.cache.get(key)
    if entity_id:
        entity_type, _ = get_target_entity(entity_path)
        entity = _create_entity_from_cache(entity_type, entity_id, stub)
        return True, entity
    # An entity_id of '' means it was cached as not found
    in_cache = entity_id is not None
    return in_cache, None


def _put_in_cache(connection, entity_path, entity):
    key = _cache_key(connection, entity_path)
    entity_id = entity._moId if entity else ''
    current_app.cache.set(key, entity_id)


def _create_entity_from_cache(entity_type, entity_id, stub):
    vim_class = ENTITY_TYPES[entity_type]
    return vim_class(entity_id, stub=stub)


def _cache_key(connection, entity_path):
    return u'{}_entity_{}'.format(
        connection.cache_key,
        u'_'.join(
            u'{}_{}'.format(entity_type, entity_name)
            for entity_type, entity_name in entity_path
        )
    )


def _find_entity(connection, entity_path):
    root = connection.content.rootFolder
    for entity_type, entity_name in entity_path:
        found = None
        for entity in list_entities(connection, root, entity_type):
            if entity.name == entity_name:
                found = entity
                break
        if not found:
            return None
        root = found
    return found


def _find_default_entity(connection):
    apitype = connection.content.about.apiType
    if apitype == API_TYPE_HOST:
        return _find_standalone_host(connection)
    elif apitype == API_TYPE_VCENTER:
        return _find_server_appliance(connection)
    else:
        raise VsphereException(u'Unrecognized API type: {}'.format(apitype))


def _find_standalone_host(connection):
    hosts = list(list_entities(
        connection,
        connection.content.rootFolder,
        'host'
    ))
    if len(hosts) != 1:
        raise VsphereException(
            u'Failed to find a single entity for the standalone host.'
        )
    return hosts[0]


def _find_server_appliance(connection):
    vms = connection.content.searchIndex.FindAllByDnsName(
        None,
        connection.host,
        True
    )
    if not vms:
        vms = connection.content.searchIndex.FindAllByIp(
            None,
            connection.host,
            True
        )
    if not vms:
        raise VsphereException(
            u'Failed to find a vCenter Server Appliance virtual machine.'
        )
    return vms[0]
