from functools import wraps

from pyVmomi import vim, vmodl
from redis import exceptions as redis_exc


class VsphereException(Exception):
    """
    Base class for the vSphere exceptions.
    """
    pass


class VsphereEntityNotFound(VsphereException):
    """
    Raised when a vSphere entity could not be found.
    """
    def __init__(self, name):
        VsphereException.__init__(
            self,
            u'Failed to find the managed entity: {}'.format(name)
        )


class VsphereEntityNotSpecified(VsphereException):
    """
    Raised when there is no entity specified. (Should be caught in validation.)
    """
    def __init__(self):
        VsphereException.__init__(
            self,
            u'No entity speficied'
        )


class VsphereVmNotPoweredOn(VsphereException):
    """
    Raised when a virtual machine is not powered on.
    """
    def __init__(self, name):
        VsphereException.__init__(
            self,
            u'The virtual machine is not powered on: {}'.format(name)
        )


class VsphereInvalidLogin(VsphereException):
    """
    Raised when a user fail to authenticate with the vSphere server.
    """
    def __init__(self, name):
        VsphereException.__init__(
            self,
            u'Failed to authenticate as the user: {}'.format(name)
        )


class VsphereUnauthorized(VsphereException):
    """
    Raised when a user doesn't have authorization to perform the query.
    """
    def __init__(self, msg):
        VsphereException.__init__(
            self,
            u'Unauthorized: {}'.format(msg)
        )


class VsphereConnectionError(VsphereException):
    """
    Raised when connecting to the vSphere server fails.
    """
    def __init__(self, error):
        VsphereException.__init__(
            self,
            u'Failed to connect to vSphere server: {}'.format(error)
        )


class VsphereCounterNotFound(VsphereException):
    """
    Raised when asking for a counter that doesn't exist.
    """
    def __init__(self, error):
        VsphereException.__init__(
            self,
            u'Counter does not exist: {}'.format(error)
        )


class VsphereCounterNotAvailable(VsphereException):
    """
    Raised when asking for a counter that doesn't exist on that entity.
    """
    def __init__(self, counter, entity):
        VsphereException.__init__(
            self,
            u'Counter {} not available on entity {}'.format(counter, entity)
        )


class VsphereCounterNotConfigured(VsphereException):
    """
    Raised when asking for a counter that's not available with the current
    configuration.
    """
    def __init__(self, counter):
        VsphereException.__init__(
            self,
            u'Counter {} not available with the current service '
            u'configuration.'.format(counter)
        )


class VsphereMissingCounterInstance(VsphereException):
    """
    Raised when an existing counter has no data for its instance.
    """
    def __init__(self, counter, instance):
        VsphereException.__init__(
            self,
            u'Counter "{}" returned no data for the "{}" instance'.format(
                counter,
                instance
            )
        )


class VsphereParameterInvalid(VsphereException):
    """
    Raised when the value of a parameter sent to vmware is invalid.
    """
    def __init__(self, error):
        VsphereException.__init__(
            self,
            u'Parameter value invalid: {}'.format(error)
        )


class VsphereNotFound(VsphereException):
    """
    Raised when a vSphere item could not be found.
    """
    def __init__(self, item_name, name):
        VsphereException.__init__(
            self,
            u'Failed to find any {} matching: {}'.format(item_name, name)
        )


class VsphereRedisError(VsphereException):
    """
    Raised when a Redis error occurrs.
    """
    def __init__(self, msg):
        VsphereException.__init__(
            self,
            u'Redis error: {}'.format(msg)
        )


def translate_exceptions(f):
    """
    Decorator that handles some common pyVmomi exceptions by translating them
    into vsphere exceptions.
    """
    @wraps(f)
    def wrapper(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except vim.fault.RestrictedByAdministrator as e:
            raise VsphereUnauthorized(e.msg)
        except vim.fault.NotAuthenticated as e:
            raise VsphereConnectionError(e.msg)
        except vmodl.fault.InvalidArgument as e:
            raise VsphereParameterInvalid(e.msg or e.invalidProperty)
        except vmodl.fault.ManagedObjectNotFound:
            # If entity is cached but doesn't exist anymore we will get this
            entity_path = kwargs.get('entity_path')
            entity_name = entity_path[-1][1] if entity_path else ''
            raise VsphereEntityNotFound(entity_name)
        except redis_exc.RedisError as e:
            raise VsphereRedisError(str(e))
        except vmodl.RuntimeFault as e:
            raise VsphereException(e.msg)
    return wrapper
