from functools import wraps
from inspect import getfullargspec
from re import search

from check_vmware.vsphere.constants import ENTITY_TYPES
from check_vmware.vsphere.exceptions import (
    VsphereCounterNotAvailable,
    VsphereCounterNotFound
)


def implemented_for(entities):
    """
    Decorator for ensuring function only called with valid entity type.
    """
    def decorator(f):
        argnames, _, _, argdefaults, *_ = getfullargspec(f)
        maxargcount = len(argnames)
        minargcount = maxargcount - len(argdefaults) if argdefaults else 0

        @wraps(f)
        def wrapper(
            connection,
            counter_name,
            entity,
            *args
        ):
            valid = [ENTITY_TYPES[vim_type] for vim_type in entities]
            if type(entity) not in valid:
                raise VsphereCounterNotAvailable(counter_name, entity.name)
            if len(args) + 2 < minargcount or len(args) + 2 > maxargcount:
                raise VsphereCounterNotFound(counter_name)
            return f(connection, entity, *args)
        wrapper.implemented_for = entities
        wrapper.inner_function = f
        return wrapper
    return decorator


def function_properties(**kwargs):
    """
    Decorator for setting custom properties on the function object.
    """
    def decorator(f):
        for k, v in kwargs.items():
            setattr(f, k, v)
        return f
    return decorator


def matches(name, regex_or_name, flags):
    """
    Match a name with respect to the given options.
    """
    if not regex_or_name:
        return True
    match = False
    for item in regex_or_name.split(','):
        item = item.strip()
        if item:
            if 'regex' in flags:
                match = search(item, name) is not None
            else:
                match = item == name
        if match:
            break
    return ('blacklist' in flags) ^ match
