from inspect import getargspec
from types import FunctionType, ModuleType

from check_vmware.vsphere import checks
from check_vmware.vsphere.counters import get_counters, get_available_counters
from check_vmware.vsphere.entities import find_entity, get_entity_type
from check_vmware.vsphere.checks import vsansdk


def list_counters(connection, entity_path=None, interval=None):
    """Fetch available counters for a given entity."""
    perf_counters = get_counters(connection.content, include_summary=True)
    entity = find_entity(
        connection,
        entity_path,
        connection.stub
    )
    metrics = get_available_counters(
        connection.content,
        entity,
        interval
    )
    counter_lookup = {
        counter['id']: counter for counter in perf_counters.values()
    }
    counters = {}
    for metric in metrics:
        if metric.counterId in counter_lookup:
            counter_name = counter_lookup[metric.counterId]['name']
            if counter_name in counters:
                counters[counter_name]['instances'].append(
                    metric.instance or '""'
                )
            else:
                counters[counter_name] = {
                    'instances': [metric.instance or '""'],
                    'summary': counter_lookup[metric.counterId]['summary']
                }
    counters.update(_special_counters(entity_type=get_entity_type(entity)))
    if vsansdk.is_vsan_sdk_enabled():
        counters.update(vsansdk.list_vsan_sdk_counters())
    return counters


def list_all_counters(connection):
    """Fetch all counters known by the vSphere server."""
    counters = get_counters(connection.content, include_summary=True)
    counters.update(_special_counters())
    return counters


def _special_counters(entity_type=None):
    counters = {}
    for module_name, module in _modules(checks):
        for function_name, function in _functions(module):
            if entity_type and entity_type not in function.implemented_for:
                continue
            inner_function = getattr(function, 'inner_function', None)
            argnames, _, _, argdefaults = getargspec(
                inner_function or function
            )
            counter_name = '{}.{}'.format(module_name, function_name)
            for i, name in enumerate(argnames[3:]):
                if argdefaults and len(argnames) - len(argdefaults) - 3 == i:
                    counter_name += '['
                counter_name += '.{}'.format(name)
            if argdefaults:
                counter_name += ']'
            summary = getattr(function, '__doc__', 'No help available.')
            summary = ' '.join(line.strip() for line in summary.split('\n'))
            counters[counter_name] = {
                'summary': summary,
                'flags': getattr(function, 'counter_flags', None)
            }
    return counters


def _sub_items(item, sub_type):
    for name in dir(item):
        if isinstance(item.__dict__.get(name), sub_type):
            yield name, item.__dict__.get(name)


def _modules(directory):
    for module in _sub_items(directory, ModuleType):
        yield module


def _functions(module):
    for function_name, function in _sub_items(module, FunctionType):
        if hasattr(function, 'implemented_for'):
            yield function_name, function
