from check_vmware.vsphere.entities import (
    find_entity,
    ensure_vm_powered_on
)
from check_vmware.vsphere.exceptions import VsphereCounterNotFound
from check_vmware.vsphere.checks import (
    runtime,
    vmfs,
    performance,
    service,
    storage,
    device,
    vsan,
    vsansdk
)


#: Dict of modules that implements checks not relying on performance counters.
special_counters = {
    'runtime': runtime,
    'vmfs': vmfs,
    'service': service,
    'storage': storage,
    'device': device,
    'vsan': vsan
}


def query(
        connection,
        counter_name,
        entity_path,
        interval,
        instance,
        flags
):
    """
    Return performance data for the given entity and counter.
    """

    entity = find_entity(
        connection,
        entity_path,
        connection.stub
    )

    (command, subcommand, args) = _split_counter(counter_name)
    module = special_counters.get(command, None)

    if module:
        func = getattr(module, subcommand, None)
        if func:
            data = func(connection, counter_name, entity, flags, *args)
            return dict(data=data)
        else:
            raise VsphereCounterNotFound(counter_name)
    elif command == 'vsansdk' and len(args):
        metric = args[0]
        vsan_sdk_counter = vsansdk.get_vsan_sdk_counter(subcommand, metric)
        if vsan_sdk_counter:
            return vsansdk.vsan_sdk_query(
                connection,
                entity,
                subcommand,
                metric,
                vsan_sdk_counter
            )
        else:
            raise VsphereCounterNotFound(counter_name)
    else:
        ensure_vm_powered_on(entity)
        return performance.query(
            connection,
            entity,
            entity_path,
            counter_name,
            interval,
            instance
        )


def _split_counter(counter_name):
    parts = counter_name.split('.', 2)
    command = parts[0]
    subcommand = parts[1] if len(parts) > 1 else ''
    arguments = parts[2:]
    return (command, subcommand, arguments)
